==> storm-sources-0.001.tar.gz: SToRM/hit_map.h <==
#ifndef HIT_MAP_H_
#define HIT_MAP_H_

#include "alignment.h"
#include "reads_data.h"
#include "reference_data.h"

/* Number of best scoring alignments remembered for each read */
#define MAP_DETAIL_SIZE 4

/* Number of successfully extended hits after which it is considered too costly to try to map the read with the default parameters */
#define MAX_MAP_UPDATES 100

#define UNMAPPED -1

/**
 * @struct HitInfoType
 * Keep trace of one read hit (score, indel positions, etc)
 * @see HitMapType
 */

typedef struct HitInfoType{
  /** the score of the alignment */
  ScoreType score;
  /** a score alignment adjusted according to the multiple alignment with the reads already mapped in that zone */
  ScoreType adjusted_score;
  /** the position in the reference where the read is mapped*/
  int ref_position;
  /** the positions of the indels in the aligned read, multiplied by 1 for insertions and -1 for deletions */
  /* The values, explained:
   *  0  => no more indels in this read
   *  N  => insertion in the read at position N-1
   * -N  => deletion  in the read at position N-1 */
  short *indel_positions;
  int sense;
  unsigned char* traceback;
  int traceback_seq_len;
} HitInfoType;



/**
 * @struct HitMapType
 * Keep trace of all the hits for all the reads
 * @see HitInfoType
 */

typedef struct HitMapType{
  int size;
  /** max number of indels */
  int indel_count;
  /** number of mapped reads */
  int mapped;
  /** 2D array 
   * (of size equal to (nb reads) x (fixed nb maxhits per read = MAP_DETAIL_SIZE )) 
   *  of HitInfoType struct
   * FIXME : check the size of this array
   */
  HitInfoType** map;
} HitMapType;

/**
 * Creates a map.
 * @param size
 * @return The address of the created map
 */
HitMapType* hit_map__create(int size, int indel_count);

/**
 * Initializes a hit map
 * @param map The map to initialize
 * @param size
 */
void hit_map__init(HitMapType* map, int size, int indel_count);
/**
 * Cleans the contents of the map
 */
inline void hit_map__clean(HitMapType* map);

/**
 * Updates a map with a new alignment
 * @param map The map to update
 * @param read_id The aligned read's index in the reads database
 * @param ref_position The position where the aligned part of the reference starts
 * @param alignment The obtained alignment
 * @return The rank of the alignment if its score was good enough to include in the map
 * (between 0 and MAP_DETAIL_SIZE exclusive), or -1 otherwise.
 */
int hit_map__update(HitMapType* map, const int read_id, const int ref_position, const AlignmentType* alignment, int alignment_sense);

/**
 * Obtain the score for a read, when mapped on a given position
 * @param map
 * @param read_id The read index
 * @param ref_position The position of interest in the reference genome
 * @param rank The rank of that score will be placed at this address
 * @return The score
 */
inline int hit_map__score_for(HitMapType* map, int read_id, int ref_position, int *rank);

/**
 * Outputs the mapped reads
 * @param map
 * @param ref_db
 * @param reads_db
 * @param output
 */
void hit_map__display__colors(HitMapType* map, ReferenceDBType* ref_db, ReadsDBType* reads_db, FILE* output, FILE* seq_output);

/**
 * Quick display of the successfully matched reads: index, position on the reference, score, alignment length, color sequence, alignment summary
 */
void hit_map__display(const HitMapType* map, const ReferenceDBType* ref_db, const ReadsDBType* reads_db);
/**
 * Cleanup
 */
void hit_map__destroy(HitMapType* map);

#endif /* HIT_MAP_H_ */

==> storm-sources-0.002.tar.gz: SToRM/hit_map.h <==
#ifndef HIT_MAP_H_
#define HIT_MAP_H_

#include "alignment.h"
#include "reads_data.h"
#include "reference_data.h"

/* Number of best scoring alignments remembered for each read */
#define MAP_DETAIL_SIZE 4 /* at least 2 */

/* Number of successfully extended hits after which it is considered too costly to try to map the read with the default parameters */
#define MAX_MAP_UPDATES 100

#define UNMAPPED -1

/**
 * @struct HitInfoType
 * Keep trace of one read hit (score, indel positions, etc)
 * @see HitMapType
 */

typedef struct HitInfoType{
  /** the score of the alignment */
  ScoreType score;
  /** a score alignment adjusted according to the multiple alignment with the reads already mapped in that zone */
  ScoreType adjusted_score;
  /** the nreference sequence number */
  int ref_id;
  /** the position in the reference where the read is mapped */
  int ref_pos;
  /** the positions of the indels in the aligned read, multiplied by 1 for insertions and -1 for deletions */
  /* The values, explained:
   *  0  => no more indels in this read
   *  N  => insertion in the read at position N-1
   * -N  => deletion  in the read at position N-1 
   */
  short *indel_positions;
  /** the alignment sense (+1/-1) */
  int sense;
  /** alignment traceback (2 symbols per Byte)*/
  unsigned char* traceback;
  /** alignment traceback symbol length */
  int traceback_seq_len;
} HitInfoType;



/**
 * @struct HitMapType
 * Keep trace of all the hits for all the reads
 * @see HitInfoType
 */

typedef struct HitMapType{
  /** number of reads */
  int size;
  /** max number of indels */
  int indel_count;
  /** number of mapped reads (stats) */
  int mapped;
  /** 2D array 
   * (of size equal to (nb reads) x (fixed nb maxhits per read = MAP_DETAIL_SIZE )) 
   *  of HitInfoType struct
   */
  HitInfoType** map;
} HitMapType;

/**
 * Creates a map.
 * @param size The number of reads
 * @param indel_count The maximum number of allowed indels per read
 * @return The address of the created map
 */
HitMapType* hit_map__create(int size, int indel_count);

/**
 * Initializes a hit map
 * @param map The map to initialize
 * @param size The number of reads
 * @param indel_count The maximum number of allowed indels per read
 */
void hit_map__init(HitMapType* map, int size, int indel_count);

/**
 * Cleans the contents of the map
 * @param map The map to clean
 */
inline void hit_map__clean(HitMapType* map);

/**
 * Updates a map with a new alignment
 * @param map The map to update
 * @param read_id The aligned read's index in the reads database
 * @param ref_id The reference id that hits the read
 * @param ref_pos The position where the aligned part of the reference starts
 * @param alignment The obtained alignment
 * @return The rank of the alignment if its score was good enough to include in the map
 * (between 0 and MAP_DETAIL_SIZE exclusive), or -1 otherwise.
 */
int hit_map__update(HitMapType* map, const int read_id, const int ref_id, const int ref_pos, const AlignmentType* alignment, int alignment_sense);

/**
 * Obtain the score for a read, when mapped on a given position
 * @param map 
 * @param read_id The read index
 * @param ref_id The reference id of interest
 * @param ref_pos The position of interest in the reference genome
 * @param rank The rank of that score will be placed at this address
 * @return The score
 */
inline int hit_map__score_for(const HitMapType* map, const int read_id, const int ref_id, const int ref_pos);

/**
 * Quick display of the successfully matched reads: index, position on the reference, score, alignment length, code sequence, alignment summary
 * @param map
 */
void hit_map__display(const HitMapType* map);

/**
 * Cleanup
 */
void hit_map__destroy(HitMapType* map);

#endif /* HIT_MAP_H_ */
